//---------------------------------------------------------------------------

#pragma hdrstop

#define _EXPORTING
#include "..\miniscope\device\MiniscopeDevice.h"
#include "..\miniscope\device\MiniscopeDeviceCapabilities.h"
#include "WaveInput.h"
#include <assert.h>

//---------------------------------------------------------------------------
#pragma package(smart_init)

static const struct S_DEVICE_DLL_INTERFACE dll_interface =
	{DLL_INTERFACE_MAJOR_VERSION, DLL_INTERFACE_MINOR_VERSION};

// callback ptrs
CALLBACK_LOG lpLogFn = NULL;
CALLBACK_CONNECT lpConnectFn = NULL;
CALLBACK_DATA lpDataFn = NULL;
CALLBACK_TRIGGER lpTriggerFn = NULL;


void *callbackCookie;	///< used by upper class to distinguish library instances when receiving callbacks

void __stdcall GetMiniscopeInterfaceDescription(struct S_DEVICE_DLL_INTERFACE* interf)
{
	interf->majorVersion = dll_interface.majorVersion;
	interf->minorVersion = dll_interface.minorVersion;
}

void Log(char* txt)
{
	if (lpLogFn)
		lpLogFn(callbackCookie, txt);
}

void Data(short* data, int size)
{
	if (lpTriggerFn)
		lpTriggerFn(callbackCookie);
	if (lpDataFn)
		lpDataFn(callbackCookie, data, size);
}

void __stdcall SetCallbacks(void *cookie, CALLBACK_LOG lpLog, CALLBACK_CONNECT lpConnect,
	CALLBACK_DATA lpData, CALLBACK_TRIGGER lpTrigger)
{
	assert(cookie && lpLog && lpConnect && lpData && lpTrigger);
	lpLogFn = lpLog;
	lpConnectFn = lpConnect;
	lpDataFn = lpData;
	lpTriggerFn = lpTrigger;
	callbackCookie = cookie;
	lpLogFn(callbackCookie, "I'm alive!");
	
	waveIn.callbackLog = Log;
	waveIn.callbackData = Data;
}

void __stdcall GetDeviceCapabilities(struct S_SCOPE_CAPABILITIES **caps)
{
	static float fSens[] = { 3.05e-5 };
	static float fSampl[] = { 2.2675737e-5 };
	static unsigned int iBuf[] = { 2048 };
	static enum E_ACQUISITION_MODE eAcqMode[] = { ACQUISITION_CONTINUOUS };
	static enum E_TRIGGER_TYPE eTriggerType[] = { TRIGGER_CONTINUOUS };
	static enum E_COUPLING_TYPE eCouplingType[] = { COUPLING_AC };
	static struct S_SCOPE_CAPABILITIES capabilities =
	{
		16,		//unsigned int iBitsPerSample;
		sizeof(fSens)/sizeof(fSens[0]), fSens,
		sizeof(fSampl)/sizeof(fSampl[0]), fSampl,
		sizeof(eTriggerType)/sizeof(eTriggerType[0]), eTriggerType,
		sizeof(eAcqMode)/sizeof(eAcqMode[0]), eAcqMode,
		sizeof(eCouplingType)/sizeof(eCouplingType[0]), eCouplingType,
		sizeof(iBuf)/sizeof(iBuf[0]), iBuf,
		0		//unsigned int iOffset;               ///< is offset setting available?
	};
	*caps = &capabilities;
}

void __stdcall ShowSettings(HANDLE parent)
{
	MessageBox(parent, "This dll has no settings.\nIt uses default WAVE IN device.", "Device DLL", MB_ICONINFORMATION);
}

int __stdcall SetSensitivity(int iId)
{
	return 0;
}

int __stdcall SetSamplingPeriod(int iId)
{
	return 0;
}

int __stdcall SetTriggerSource(int iId)
{
	return 0;
}

int __stdcall SetAcqMode(int iId)
{
	return 0;
}

int __stdcall SetCouplingType(int iId)
{
	return 0;
}

int __stdcall SetBufferSize(int iId)
{
	return 0;
}

int __stdcall SetOffset(float fOffset)
{
	return 0;
}

int __stdcall Connect(void)
{
	return waveIn.Connect();
}

int __stdcall Disconnect(void)
{
	return waveIn.Disconnect();
}

int __stdcall AcquireStart(void)
{
	return waveIn.RecStart();
}

int __stdcall AcquireStop(void)
{
	return waveIn.RecStop();
}

int __stdcall ManualTrigger(void)
{
	return 0;
}

